local addonName, addonTable = ...
local L = addonTable.L
local E = addonTable.E
local tinsert = _G.tinsert
local tremove = _G.tinsert

local pairs = _G.pairs
local ipairs = _G.ipairs
local wipe = _G.wipe
local select = _G.select
local table_sort = _G.table.sort
local format = _G.format
local CreateFrame = _G.CreateFrame
local GetSpecialization = _G.GetSpecialization
local GetSpecializationInfo = _G.GetSpecializationInfo
local GetTalentInfo = _G.GetTalentInfo

local equipmentSets
local multiSpecPaperDollPane
local paperDollSidebarTab4
local initialized = false

E.ui = {}

local function checkTalents(name)
    if E.equipmentSets[name].switchTalents then
        for k, v in ipairs(E.equipmentSets[name].talents) do
            local _, _, _, learned = GetTalentInfo(k, v, 1)
            if not learned then
                return false
            end
        end
    end
    return true
end

local function gatherEquipmentSets()
    E:gatherEquipmentSets()
    local i = 0
    if not equipmentSets then
        equipmentSets = {}
    else
        wipe(equipmentSets)
    end

    for k, v in pairs(E.equipmentSets) do
        i = i + 1
        equipmentSets[i] = {k, v.specialization, v.texture, v.missingItems, v.isEquipped, v.switchTalents}
    end
    table_sort(equipmentSets, function(a, b)
        return a[1]<b[1]
    end)
end

local function buttonUpdate(button, scrollOffset, i)
    button:Show()
    button.text:SetText(equipmentSets[i+scrollOffset][1])
    button.value = equipmentSets[i+scrollOffset][1]
    button.icon:SetTexture(equipmentSets[i+scrollOffset][3] or 134400)
    if equipmentSets[i+scrollOffset][4] > 0 then
        button.text:SetTextColor(_G.RED_FONT_COLOR.r, _G.RED_FONT_COLOR.g, _G.RED_FONT_COLOR.b)
        button:Disable()
    else
        button.text:SetTextColor(_G.NORMAL_FONT_COLOR.r, _G.NORMAL_FONT_COLOR.g, _G.NORMAL_FONT_COLOR.b);
        button:Enable()
    end

    if equipmentSets[i+scrollOffset][5] and (equipmentSets[i+scrollOffset][2] == 0 or (equipmentSets[i+scrollOffset][2] == GetSpecialization() and checkTalents(equipmentSets[i+scrollOffset][1]))) then
        button.Check:Show()
    else
        button.Check:Hide()
    end

    if button.value == multiSpecPaperDollPane.selectedEquipSet then
        button.SelectedBar:Show()
        if button.Check:IsShown() then
            multiSpecPaperDollPane.EquipSet:Disable()
        else
            multiSpecPaperDollPane.EquipSet:Enable()
        end
    else
        button.SelectedBar:Hide()
    end

    if ((i+scrollOffset) == 1) then
        button.BgTop:Show();
        button.BgMiddle:SetPoint("TOP", button.BgTop, "BOTTOM");
    else
        button.BgTop:Hide();
        button.BgMiddle:SetPoint("TOP");
    end

    if ((i+scrollOffset) == #equipmentSets) then
        button.BgBottom:Show();
        button.BgMiddle:SetPoint("BOTTOM", button.BgBottom, "TOP");
    else
        button.BgBottom:Hide();
        button.BgMiddle:SetPoint("BOTTOM");
    end
    if ((i+scrollOffset)%2 == 0) then
        button.Stripe:SetColorTexture(0.9, 0.9, 1);
        button.Stripe:SetAlpha(0.1);
        button.Stripe:Show();
    else
        button.Stripe:Hide();
    end
end

local function scrollPaneUpdate(frame)
    gatherEquipmentSets()
    _G.HybridScrollFrame_Update(frame, #equipmentSets * frame.buttons[1]:GetHeight() + frame.EquipSet:GetHeight() + 20 , frame:GetHeight());
    local scrollOffset = _G.HybridScrollFrame_GetOffset(frame);
    for i=1, #frame.buttons do
        local button = frame.buttons[i]
        if i+scrollOffset <= #equipmentSets then
            buttonUpdate(button, scrollOffset, i)
        else
            button:Hide()
        end
    end
end

local function initScrollPaneButton(button, i)
    button:SetScript("OnEnter", function(self)
        _G.GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        local text = L["No specialization assigned"]
        if equipmentSets[i][2]>0 then
            text = format(L["Specialization: %s"], E.specTable[equipmentSets[i][2]].name)
        end
        if not self:IsEnabled() then
            text = text .. format(L["\n%s items missing"], equipmentSets[i][4])
        end
        _G.GameTooltip:SetText(text, 1, 1, 1, true);
        self.HighlightBar:Show();
    end)
    button:SetScript("OnLeave", function(self)
        _G.GameTooltip:Hide();
        button.HighlightBar:Hide();
    end)
    button:SetScript("OnClick", function(self)
        multiSpecPaperDollPane.selectedEquipSet = self.value
        scrollPaneUpdate(multiSpecPaperDollPane)
    end)
end

function E.ui:isInitialized()
    return initialized
end

function E.ui:show()
    paperDollSidebarTab4:Show()
    paperDollSidebarTab4:SetWidth(33)
    _G.PaperDollSidebarTab3:SetPoint("RIGHT", paperDollSidebarTab4, "LEFT", -4, 0)
end

function E.ui:hide()
    paperDollSidebarTab4:Hide()
    paperDollSidebarTab4:SetWidth(0)
    _G.PaperDollSidebarTab3:SetPoint("BOTTOMRIGHT", _G.PaperDollSidebarTab3:GetParent(), "BOTTOMRIGHT", -30, 0)
end

function E.ui:init()
    paperDollSidebarTab4 = CreateFrame("Button", "PaperDollSidebarTab4", _G.PaperDollSidebarTabs)
    paperDollSidebarTab4.id = 4
    paperDollSidebarTab4.GetID = function(self)
        return self.id
    end
    paperDollSidebarTab4:SetWidth(33)
    paperDollSidebarTab4:SetHeight(35)
    paperDollSidebarTab4:SetPoint("BOTTOMRIGHT", 0, 0)

    paperDollSidebarTab4.TabBg = paperDollSidebarTab4:CreateTexture(nil, "BACKGROUND")
    paperDollSidebarTab4.TabBg:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    paperDollSidebarTab4.TabBg:SetWidth(50)
    paperDollSidebarTab4.TabBg:SetHeight(43)
    paperDollSidebarTab4.TabBg:SetTexCoord(0.01562500, 0.79687500, 0.61328125, 0.78125000)
    paperDollSidebarTab4.TabBg:SetPoint("BOTTOMLEFT", -9, -2)

    paperDollSidebarTab4.Icon = paperDollSidebarTab4:CreateTexture(nil, "ARTWORK")
    paperDollSidebarTab4.Icon:SetTexture(select(4, GetSpecializationInfo(GetSpecialization())))
    paperDollSidebarTab4.Icon:SetTexCoord(.08, .92, .08, .92)

    paperDollSidebarTab4.Icon:SetWidth(27)
    paperDollSidebarTab4.Icon:SetHeight(28)
    paperDollSidebarTab4.Icon:SetPoint("BOTTOM", 1, 1)

    paperDollSidebarTab4.Hider = paperDollSidebarTab4:CreateTexture(nil, "OVERLAY")
    paperDollSidebarTab4.Hider:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    paperDollSidebarTab4.Hider:SetWidth(34)
    paperDollSidebarTab4.Hider:SetHeight(19)
    paperDollSidebarTab4.Hider:SetTexCoord(0.01562500, 0.54687500, 0.11328125, 0.18750000)
    paperDollSidebarTab4.Hider:SetPoint("BOTTOM")

    paperDollSidebarTab4.Highlight = paperDollSidebarTab4:CreateTexture(nil, "HIGHLIGHT")
    paperDollSidebarTab4.Highlight:SetTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
    paperDollSidebarTab4.Highlight:SetTexCoord(0.01562500, 0.50000000, 0.19531250, 0.31640625)
    paperDollSidebarTab4.Highlight:SetWidth(31)
    paperDollSidebarTab4.Highlight:SetHeight(31)
    paperDollSidebarTab4.Highlight:SetPoint("TOPLEFT", 2, -3)

    paperDollSidebarTab4:SetScript("OnEnter", function(self)
        _G.GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        _G.GameTooltip:SetText(_G.PAPERDOLL_SIDEBARS[self:GetID()].name, 1, 1, 1);
    end)

    paperDollSidebarTab4:SetScript("OnLeave", function(self)
        _G.GameTooltip:Hide();
    end)

    paperDollSidebarTab4:SetScript("OnClick", function(self)
        _G.PaperDollFrame_SetSidebar(self, self:GetID());
    end)


    multiSpecPaperDollPane = CreateFrame("ScrollFrame", "MultiSpecPaperDollPane", _G.PaperDollSidebarTabs, "HybridScrollFrameTemplate")
    multiSpecPaperDollPane:SetWidth(172)
    multiSpecPaperDollPane:SetHeight(354)
    multiSpecPaperDollPane:SetPoint("TOPLEFT", _G.CharacterFrameInsetRight, 4, -4)
    multiSpecPaperDollPane:SetFrameLevel(_G.CharacterFrameInsetRight:GetFrameLevel()+1)
    multiSpecPaperDollPane.scrollBar = CreateFrame("Slider", "MultiSpecPaperDollPaneScrollBar", multiSpecPaperDollPane, "HybridScrollBarTemplate")
    multiSpecPaperDollPane.scrollBar.doNotHide = 1
    multiSpecPaperDollPane.scrollBar:SetPoint("TOPLEFT", multiSpecPaperDollPane, "TOPRIGHT", 4, -13)
    multiSpecPaperDollPane.scrollBar:SetPoint("BOTTOMLEFT", multiSpecPaperDollPane, "BOTTOMRIGHT", 4, 14)
    multiSpecPaperDollPane.EquipSet = CreateFrame("Button", nil, multiSpecPaperDollPane, "UIPanelButtonTemplate")
    multiSpecPaperDollPane.EquipSet:SetWidth(174)
    multiSpecPaperDollPane.EquipSet:SetHeight(22)
    multiSpecPaperDollPane.EquipSet:SetPoint("TOPLEFT", 0, 0)
    multiSpecPaperDollPane.EquipSet.ButtonBackground = multiSpecPaperDollPane.EquipSet:CreateTexture()
    multiSpecPaperDollPane.EquipSet.ButtonBackground:SetPoint("TOPLEFT", multiSpecPaperDollPane)
    multiSpecPaperDollPane.EquipSet.ButtonBackground:SetPoint("BOTTOMRIGHT", multiSpecPaperDollPane, "TOPRIGHT", 0, -23)
    multiSpecPaperDollPane.EquipSet.ButtonBackground:SetTexture(0, 0, 0)
    multiSpecPaperDollPane.EquipSet:SetText("Activate")
    multiSpecPaperDollPane.EquipSet:SetScript("OnClick", function(self)
        if multiSpecPaperDollPane.selectedEquipSet then
            E:changeTalentsAndEquipSet(multiSpecPaperDollPane.selectedEquipSet)
        end
    end)
    multiSpecPaperDollPane.hookFrame = CreateFrame("Frame", nil, multiSpecPaperDollPane)
    multiSpecPaperDollPane.hookFrame:SetScript("OnShow", function(self)
        multiSpecPaperDollPane.hookFrame:RegisterEvent("EQUIPMENT_SWAP_FINISHED")
        multiSpecPaperDollPane.hookFrame:RegisterEvent("PLAYER_TALENT_UPDATE")
        multiSpecPaperDollPane.hookFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        multiSpecPaperDollPane.hookFrame:RegisterEvent("EQUIPMENT_SETS_CHANGED")
        multiSpecPaperDollPane.hookFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
        scrollPaneUpdate(multiSpecPaperDollPane)
    end)
    multiSpecPaperDollPane.hookFrame:SetScript("OnHide", function(self)
        multiSpecPaperDollPane.hookFrame:UnregisterEvent("EQUIPMENT_SWAP_FINISHED")
        multiSpecPaperDollPane.hookFrame:UnregisterEvent("PLAYER_TALENT_UPDATE")
        multiSpecPaperDollPane.hookFrame:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
        multiSpecPaperDollPane.hookFrame:UnregisterEvent("EQUIPMENT_SETS_CHANGED")
        multiSpecPaperDollPane.hookFrame:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
    end)

    multiSpecPaperDollPane.hookFrame:SetScript("OnEvent", function(self, event, ...)
        if event == "EQUIPMENT_SWAP_FINISHED"
        or event == "PLAYER_TALENT_UPDATE"
        or event == "EQUIPMENT_SETS_CHANGED"
        or event == "PLAYER_EQUIPMENT_CHANGED"
        or event == "ACTIVE_TALENT_GROUP_CHANGED" then
            scrollPaneUpdate(multiSpecPaperDollPane)
            paperDollSidebarTab4.Icon:SetTexture(select(4, GetSpecializationInfo(GetSpecialization())))
        end
    end)
    _G.HybridScrollFrame_OnLoad(multiSpecPaperDollPane)
    multiSpecPaperDollPane.update = scrollPaneUpdate
    --multiSpecPaperDollPane:SetHeight(600)
    _G.HybridScrollFrame_CreateButtons(multiSpecPaperDollPane, "GearSetButtonTemplate", 2, -(multiSpecPaperDollPane.EquipSet:GetHeight()+4))

    gatherEquipmentSets()
    for i=1, #multiSpecPaperDollPane.buttons do
        initScrollPaneButton(multiSpecPaperDollPane.buttons[i], i)
    end
    multiSpecPaperDollPane:Hide()

    tinsert(_G.PAPERDOLL_SIDEBARS, {
        name="MultiSpec";
        frame="MultiSpecPaperDollPane";
        icon = "Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs";
        texCoords = {0.01562500, 0.53125000, 0.46875000, 0.60546875};
    })

    initialized = true
end
